/*
 * Copyright (C) 2003-2005 Atmark Techno, Inc.  All Rights Reserved.
 */

#include "flash_region.h"
#include "memregions.h"
#include <target/flash.h>
#include "mode.h"

struct FlashRegion {
        enum FlashRegionID id;
        unsigned int start;
        unsigned int size;
        unsigned int end;
        char * name;
        char * blocking;
};

struct FlashRegion flash_regions_8MiB[] = {
        { .id       = FlashRegionIdFpga,
          .start    = FLASH_8MiB_FPGA_START,
          .size     = FLASH_8MiB_FPGA_SIZE,
          .end      = FLASH_8MiB_FPGA_END,
          .name     = "fpga",
          .blocking = FLASH_8MiB_FPGA_BLOCKING,
        },
        { .id       = FlashRegionIdBootloader,
          .start    = FLASH_8MiB_BOOTLOADER_START,
          .size     = FLASH_8MiB_BOOTLOADER_SIZE,
          .end      = FLASH_8MiB_BOOTLOADER_END,
          .name     = "bootloader",
          .blocking = FLASH_8MiB_BOOTLOADER_BLOCKING,
        },
        { .id       = FlashRegionIdImage,
          .start    = FLASH_8MiB_IMAGE_START,
          .size     = FLASH_8MiB_IMAGE_SIZE,
          .end      = FLASH_8MiB_IMAGE_END,
          .name     = "image",
          .blocking = FLASH_8MiB_IMAGE_BLOCKING,
        },
        { .id       = FlashRegionIdKernel,
          .start    = FLASH_8MiB_KERNEL_START,
          .size     = FLASH_8MiB_KERNEL_SIZE,
          .end      = FLASH_8MiB_KERNEL_END,
          .name     = "kernel",
          .blocking = FLASH_8MiB_KERNEL_BLOCKING,
        },
        { .id       = FlashRegionIdUser,
          .start    = FLASH_8MiB_USER_START,
          .size     = FLASH_8MiB_USER_SIZE,
          .end      = FLASH_8MiB_USER_END,
          .name     = "user",
          .blocking = FLASH_8MiB_USER_BLOCKING,
        },
        { .id       = FlashRegionIdConfig,
          .start    = FLASH_8MiB_CONFIG_START,
          .size     = FLASH_8MiB_CONFIG_SIZE,
          .end      = FLASH_8MiB_CONFIG_END,
          .name     = "config",
          .blocking = FLASH_8MiB_CONFIG_BLOCKING,
        },
        { .id       = FlashRegionIdFree1,
          .start    = FLASH_8MiB_FREE1_START,
          .size     = FLASH_8MiB_FREE1_SIZE,
          .end      = FLASH_8MiB_FREE1_END,
          .name     = "free1",
          .blocking = FLASH_8MiB_FREE1_BLOCKING,
        },
        { .id       = FlashRegionIdFree2,
          .start    = FLASH_8MiB_FREE2_START,
          .size     = FLASH_8MiB_FREE2_SIZE,
          .end      = FLASH_8MiB_FREE2_END,
          .name     = "free2",
          .blocking = FLASH_8MiB_FREE2_BLOCKING,
        },
};

struct FlashRegion flash_regions_4MiB[] = {
        { .id       = FlashRegionIdFpga,
          .start    = FLASH_4MiB_FPGA_START,
          .size     = FLASH_4MiB_FPGA_SIZE,
          .end      = FLASH_4MiB_FPGA_END,
          .name     = "fpga",
          .blocking = FLASH_4MiB_FPGA_BLOCKING,
        },
        { .id       = FlashRegionIdBootloader,
          .start    = FLASH_4MiB_BOOTLOADER_START,
          .size     = FLASH_4MiB_BOOTLOADER_SIZE,
          .end      = FLASH_4MiB_BOOTLOADER_END,
          .name     = "bootloader",
          .blocking = FLASH_4MiB_BOOTLOADER_BLOCKING,
        },
        { .id       = FlashRegionIdImage,
          .start    = FLASH_4MiB_IMAGE_START,
          .size     = FLASH_4MiB_IMAGE_SIZE,
          .end      = FLASH_4MiB_IMAGE_END,
          .name     = "image",
          .blocking = FLASH_4MiB_IMAGE_BLOCKING,
        },
        { .id       = FlashRegionIdKernel,
          .start    = FLASH_4MiB_KERNEL_START,
          .size     = FLASH_4MiB_KERNEL_SIZE,
          .end      = FLASH_4MiB_KERNEL_END,
          .name     = "kernel",
          .blocking = FLASH_4MiB_KERNEL_BLOCKING,
        },
        { .id       = FlashRegionIdUser,
          .start    = FLASH_4MiB_USER_START,
          .size     = FLASH_4MiB_USER_SIZE,
          .end      = FLASH_4MiB_USER_END,
          .name     = "user",
          .blocking = FLASH_4MiB_USER_BLOCKING,
        },
        { .id       = FlashRegionIdConfig,
          .start    = FLASH_4MiB_CONFIG_START,
          .size     = FLASH_4MiB_CONFIG_SIZE,
          .end      = FLASH_4MiB_CONFIG_END,
          .name     = "config",
          .blocking = FLASH_4MiB_CONFIG_BLOCKING,
        },
        { .id       = FlashRegionIdFree1,
          .start    = 0,
          .size     = 0,
          .end      = 0,
          .name     = "free1",
          .blocking = "0x0K",
        },
        { .id       = FlashRegionIdFree2,
          .start    = 0,
          .size     = 0,
          .end      = 0,
          .name     = "free2",
          .blocking = "0x0K",
        },
};

static int is_flash_8MiB(void)
{
        return ((flash_get_size(FLASH_BASEADDR) == FLASH_8MiB) && (!is_compatible_mode()));
}

#define flash_region_get(id, attr) (is_flash_8MiB() ? flash_regions_8MiB[id].attr : flash_regions_4MiB[id].attr)
#define NULL ((void *)0)

unsigned int flash_region_get_start_address(enum FlashRegionID id)
{
        unsigned int ret;

        if (id > FlashRegionIdMax)
                ret = 0xffffffff;

        return flash_region_get(id, start);
}

unsigned int flash_region_get_end_address(enum FlashRegionID id)
{
        unsigned int ret;

        if (id > FlashRegionIdMax)
                ret = 0x0;

        return flash_region_get(id, end);
}

int flash_region_get_size(enum FlashRegionID id)
{
        unsigned int ret;

        if (id > FlashRegionIdMax)
                ret = -1;

        return flash_region_get(id, size);
}

char * flash_region_get_name(enum FlashRegionID id)
{
        char * ret;

        if (id > FlashRegionIdMax)
                ret = NULL;

        return flash_region_get(id, name);
}

char * flash_region_get_blocking(enum FlashRegionID id)
{
        char * ret;

        if (id > FlashRegionIdMax)
                ret = NULL;

        return flash_region_get(id, blocking);
}
